      PROGRAM EX0406
      DIMENSION TB(3),TC(3),PC(3)
      DATA WMOL,T,P/0.85,288.15,6.E+6/
      DATA TB(1),TC(1),PC(1)/111.6,190.55,4.599E+6/
      DATA TB(2),TC(2),PC(2)/184.6,305.33,4.871E+6/
      CALL PITZCURL(T,P,WMOL,TB,TC,PC,D)
      WRITE (*,1000) D
 1000 FORMAT(1X,'DENSITY [KMOL/M3] = ',F5.3)
      END


      SUBROUTINE PITZCURL(T,P,WMOL,TB,TC,PC,D)
C
C     The Subroutine calculates the density of binary mixtures of hydro-
C     carbon gases using a three-form truncated virial equation.
C     The coefficients are expressed in the Pitzer-Curl form and
C     are calculated from the critical properties, and the acentric
C     factor, OMEG. The acentric factor is calculated from the
C     Lee-Kesler equation in terms of the critical constants and the
C     boiling temperature.
C     - For the cross-coefficients the Lorentz-Berthelot mixing rules
C       are employed.
C     - Data required are: mole fraction of component 1 (WMOL), for each
C       component normal boiling temperature, TB(I) in [K], critical
C       temperature, TC(I) in [K], and critical pressure, PC(I) in [Pa]
C
      DIMENSION OMEG(3),B(3),C(3),TB(3),TC(3),PC(3)
      DATA R,DEN/8.3145,100./
C
C     Calculation of Acentric Factors
C
      DO 100 I=1,2
      TBR=TB(I)/TC(I)
      A1=LOG(0.10133E+6/PC(I))-5.92714+6.09648/TBR+1.28862*LOG(TBR)
     +-0.169347*TBR**6
      A2=15.2518-15.6875/TBR-13.4721*LOG(TBR)+0.43577*TBR**6
  100 OMEG(I)=A1/A2
C
C     Mixing Rules
C
      OMEG(3)=(OMEG(1)+OMEG(2))/2.
      VC1=(0.291-0.08*OMEG(1))*R*TC(1)/PC(1)
      VC2=(0.291-0.08*OMEG(2))*R*TC(2)/PC(2)
      VC=(((VC1**(1./3.))+(VC2**(1./3.)))**3)/8.
      WK=1.-((VC1*VC2)**.5)/VC
      TC(3)=(1.-WK)*((TC(1)*TC(2))**.5)
      PC(3)=(0.291-0.04*(OMEG(1)+OMEG(2)))*R*TC(3)/VC
C
C     Calculation of Virial Coefficients (BB and CC)
C
      DO 200 I=1,3
      TR=T/TC(I)
      B0=0.1445-(0.33/TR)-0.1385*(TR**(-2))-0.0121*(TR**(-3))-
     &0.000607*(TR**(-8))
      B1=0.0637+0.331*(TR**(-2))-0.423*(TR**(-3))-0.008*(TR**(-8))
      B(I)=(R*TC(I)/PC(I))*(B0+OMEG(I)*B1)
      C0=0.01407+0.02432*(TR**(-2.8))-0.00313*(TR**(-10.5))
      C1=-0.02676+0.0177*(TR**(-2.8))+0.04*(TR**(-3))-0.003*(TR**(-6))
     &-0.00228*(TR**(-10.5))
  200 C(I)=((R*TC(I)/PC(I))**2)*(C0+OMEG(I)*C1)
      XMOL=1.-WMOL
      BB=(B(1)*WMOL**2)+(B(2)*XMOL**2)+(2.*WMOL*XMOL*B(3))
      CC=(C(1)*WMOL**3)+(C(2)*XMOL**3)+(3.*WMOL*XMOL*C(3))
C
C     Calculation of Density (D)
C
  300 D=P/(R*T*(1+BB*DEN+CC*DEN*DEN))
      CHK=ABS(D-DEN)*100./D
      IF (CHK.LT.0.01) GO TO 400
      DEN=D
      GO TO 300
  400 D=D/1000.
      RETURN
      END
