      PROGRAM EX0407
      DATA T,P/280.,60./
      DATA C1,C2,C3,C4,C5,C6,C7,C8/0.735015,0.033152,0.007657,
     &0.002458,0.00074,0.000241,0.000148,0.000070/
      DATA CN2,CO2,H2,CO,HE/0.100214,0.016245,
     &0.094918,0.009142,0.0/
      CALL VGERG(T,P,CF,DE,C1,CN2,CO2,C2,H2,C3,CO,C4,HE,C5,C6,C7,C8)
      WRITE (*,1000) CF,DE
 1000 FORMAT(1X,'COMPRESSION FACTOR : ',F8.5,/1X,'DENSITY [MOL/LT]   : '
     &,F8.5)
      END

      SUBROUTINE VGERG
     &(T,P,CF,DE,C1,CN2,CO2,C2,H2,C3,CO,C4,HE,C5,C6,C7,C8)
C
C     Calculation of Density of Natural Gas Mixture -up to 13 components
C     according to: Jaeschke et al. SPE Prod. Eng., Aug. 343 (1991)
C     - Limits of mole fractions [%]
C           Methane                   (C1)  >= 50  %
C           Ethane                    (C2)  <= 20  %
C           Propane                   (C3)  <=  5  %
C           Butanes                   (C4)  <=  1.5%
C           Pentanes + Benzene        (C5)  <=  0.5%
C           Hexanes + Ethylbenzene    (C6)  <=  0.1%
C           Heptanes + Toluene        (C7)  <=  0.1%
C           Octanes + Higher          (C8)  <=  0.1%
C           Nitrogen                 (CN2)  <= 50  %
C           Carbon Dioxide + Ethene  (CO2)  <= 30  %
C           Hydrogen                  (H2)  <= 10  %
C           Carbon Monoxide           (CO)  <=  3  %
C           Helium                    (HE)  <=  0.5%
C     - Temperature Range [T]   265 - 335 [K]
C     - Pressure Range    [P]     0 - 120 [BARS]
C     Expected Accuracy 0.1%
C
      DIMENSION B(37),C(62)
      COMMON /COEFF/ X(111),Y(186)
      R=.0831451
      DO 100 I=1,37
  100 B(I)=X(I)+X(37+I)*T+X(74+I)*T**2
      DO 120 J=1,62
  120 C(J)=Y(J)+Y(62+J)*T+Y(124+J)*T**2
      BB=C1**2*B(1)+CN2**2*B(13)+CO2**2*B(20)+C2**2*B(26)+H2**2*B(31)+
     &C3**2*B(32)
      BB=BB+CO**2*B(34)+C4**2*B(35)+HE**2*B(36)+C5**2*B(37)
      BB=BB+2*C1*(CN2*B(2)+CO2*B(3)+C2*B(4)+H2*B(5)+C3*B(6)
     &+CO*B(7)+C4*B(8))
      BB=BB+2*C1*(C5*B(9)+C6*B(10)+C7*B(11)+C8*B(12))
      BB=BB+2*CN2*(CO2*B(14)+C2*B(15)+H2*B(16)+C3*B(17)
     &+CO*B(18)+C4*B(19))
      BB=BB+2*CO2*(C2*B(21)+H2*B(22)+C3*B(23)+C4*B(24)+C5*B(25))
      BB=BB+2*C2*(H2*B(27)+C3*B(28)+C4*B(29)+C5*B(30))+2*C3*C4*B(33)
      CC=C1**3*C(1)+CN2**3*C(28)+CO2**3*C(45)+C2**3*C(54)+H2**3*C(59)
     &+C3**3*C(60)
      CC=CC+CO**3*C(61)+HE**3*C(62)
      CC=CC+3*C1**2*(CN2*C(2)+CO2*C(3)+C2*C(4)+H2*C(5)+C3*C(6)+CO*C(7)
     &+C4*C(8))
      CC=CC+3*C1**2*(C5*C(9)+C6*C(10)+C7*C(11))
      CC=CC+3*CN2**2*(C1*C(12)+CO2*C(29)+C2*C(30)+H2*C(31)+C3*C(32)
     &+CO*C(33)+C4*C(34))
      CC=CC+3*CO2**2*(C1*C(17)+CN2*C(35)+C2*C(46)+H2*C(47)+C3*C(48))
      CC=CC+3*C2**2*(C1*C(20)+CN2*C(39)+CO2*C(49)+H2*C(55)+C3*C(56))
      CC=CC+3*H2**2*(C1*C(25)+CN2*C(42)+CO2*C(52)+C2*C(57))
      CC=CC+3*C3**2*(C1*C(26)+CN2*C(43)+CO2*C(53)+C2*C(58))
     &+3*CO**2*(C1*C(27)+CN2*C(44))
      CC=CC+6*C1*CN2*(CO2*C(13)+C2*C(14)+H2*C(15)+C3*C(16))
      CC=CC+6*C1*CO2*(C2*C(18)+C3*C(19))
      CC=CC+6*C1*C2*(H2*C(21)+C3*C(22)+C4*C(23)+C5*C(24))
      CC=CC+6*CN2*CO2*(C2*C(36)+H2*C(37)+C3*C(38))
      CC=CC+6*CN2*C2*(H2*C(40)+C3*C(41))
      CC=CC+6*CO2*C2*(H2*C(50)+C3*C(51))
C
C     Iterations to find Z
C
      M=1
      ZF=1.
  200 DE=P/(ZF*R*T)
      Z=1.+BB*DE+CC*DE*DE
      IF (ABS(ZF-Z).LT.0.000001) GOTO 240
      M=M+1
      ZF=Z
      GOTO 200
  240 CF=Z
      END

      BLOCK DATA
      COMMON /COEFF/ X(111),Y(186)
      DATA (X(I),I=1,28)/     -.298675E+0,-.213606E+0,-.356120E+0,
     &-.499337E+0,-.328913E-1,-.554110E+0,-.687290E-1,-.138708E+1,
     &-.771367E+0,-.775140E+0,-.209989E+1,-.239409E+1,-.144600E+0,
     &-.339693E+0,-.263553E+0,+.184506E-1,-.431268E+0,-.122189E+0,
     &+.907220E-1,-.868340E+0,-.112000E+1,-.757226E-1,-.126372E+1,
     &-.188108E+1,-.202135E+1,-.107320E+1,-.174834E+0,-.142020E+1/
      DATA (X(I),I=29,56)/    -.283644E+1,-.331426E+1,-.110596E-2,
     &-.259920E+1,-.435393E+1,-.130820E+0,-.708016E+1,+.206740E-1,
     &-.111580E+2,+.133425E-2,+.104585E-2,+.144963E-2,+.205755E-2,
     &+.158947E-3,+.185897E-2,-.239381E-5,+.689575E-2,+.143827E-2,
     &+.184050E-2,+.979707E-2,+.112199E-1,+.740910E-3,+.161176E-2,
     &+.106233E-2,-.111895E-3,+.184597E-2,+.521240E-3,-.198016E-2/
      DATA (X(I),I=57,84)/    +.403760E-2,+.576913E-2,+.356052E-3,
     &+.584661E-2,+.926431E-2,+.954081E-2,+.464810E-2,+.103728E-2,
     &+.575230E-2,+.136800E-1,+.152019E-1,+.813385E-4,+.119650E-1,
     &+.213786E-1,+.602540E-3,+.363100E-1,-.513060E-4,+.535740E-1,
     &-.159761E-5,-.131159E-5,-.158061E-5,-.230786E-5,-.916521E-7,
     &-.147793E-5,+.518195E-6,-.940171E-5,+.125744E-5,-.400000E-6/
      DATA (X(I),I=85,111)/   -.125373E-4,-.143976E-4,-.911950E-6,
     &-.204429E-5,-.110155E-5,+.303122E-6,-.208060E-5,-.437181E-6,
     &+.456313E-5,-.516570E-5,-.813744E-5,-.357678E-6,-.747500E-5,
     &-.124525E-4,-.121639E-4,-.560520E-5,-.150332E-5,-.640000E-5,
     &-.181244E-4,-.189007E-4,-.987220E-7,-.152910E-4,-.287483E-4,
     &-.644300E-6,-.503829E-4,+.724000E-7,-.684970E-4/
      DATA (Y(I),I=1,28)/     +.927260E-2,+.889649E-2,+.117576E-1,
     &+.160923E-1,+.233159E-3,+.892850E-2,+.736748E-2,+.227707E+0,
     &+.320344E-1,+.320344E-1,+.320344E-1,+.874115E-2,+.900338E-2,
     &+.449508E-2,+.360589E-2,-.187767E-1,+.877002E-2,-.176574E-2,
     &-.623055E-1,-.827905E-2,-.116095E-2,-.189480E+0,-.189480E+0,
     &-.189480E+0,-.308914E-2,-.848395E-1,+.436399E-2,+.784980E-2/
      DATA (Y(I),I=29,56)/    +.552066E-2,-.136584E-1,-.650954E-2,
     &+.674630E-2,+.615912E-2,-.101202E+0,+.358783E-2,-.768174E-5,
     &+.269108E-2,-.301719E-1,-.369750E-1,-.297743E-3,-.506529E-1,
     &-.877711E-2,-.859670E-1,+.418924E-2,+.205130E-2,+.154623E+0,
     &+.843475E-2,-.594074E-1,+.133240E+0,-.516317E-2,-.944235E-1,
     &+.810002E-2,-.149820E+0,-.621000E-1,+.843800E-2,-.144234E+0/
      DATA (Y(I),I=57,84)/    +.294870E-1,-.216734E+0,+.104711E-2,
     &-.270290E+0,+.190870E-2, .000      ,-.376132E-4,-.410784E-4,
     &-.447615E-4,-.638586E-4,+.102923E-4,+.190637E-4,-.276578E-4,
     &-.139926E-2,-.691674E-4,-.691674E-4,-.691674E-4,-.443880E-4,
     &-.355487E-4,+.530233E-6,-.145437E-4,+.142145E-3,-.167108E-4,
     &+.597684E-4,+.459485E-3,+.121913E-3,+.263812E-4,+.139040E-2/
      DATA (Y(I),I=85,112)/   +.139040E-2,+.139040E-2,+.254973E-4,
     &+.605618E-3,-.100358E-4,-.398950E-4,-.168609E-4,+.106972E-3,
     &+.519104E-4,-.365557E-5,-.272612E-4,+.723716E-3,+.806674E-5,
     &+.385226E-4,-.596904E-5,+.222387E-3,+.289620E-3,+.168641E-4,
     &+.363400E-3,+.639925E-4,+.579776E-3,-.126365E-4,+.348880E-4,
     &-.965716E-3,-.355786E-4,+.430798E-3,-.807996E-3,+.579276E-4/
      DATA (Y(I),I=113,140)/  +.671721E-3,-.484242E-4,+.994385E-3,
     &+.508050E-3,-.265166E-4,+.101275E-2,-.197083E-3,+.143604E-2,
     &-.364887E-5,+.171040E-2,+.420040E-5, .000      ,+.493066E-7,
     &+.603723E-7,+.505481E-7,+.779273E-7,-.227208E-7,-.979363E-7,
     &+.343051E-7,+.221564E-5, .000      , .000      , .000      ,
     &+.696584E-7,+.467680E-7,-.152576E-7,+.204722E-7,-.221855E-6/
      DATA (Y(I),I=141,168)/  -.151577E-9,-.123785E-6,-.733765E-6,
     &-.239958E-6,-.512600E-7,-.238860E-5,-.238860E-5,-.238860E-5,
     &-.439625E-7,-.954102E-6,+.660155E-8,+.611870E-7,+.157169E-7,
     &-.179369E-6,-.887528E-7,-.234530E-7,+.391226E-7,-.121618E-5,
     &-.325798E-7,-.817506E-7,+.540142E-8,-.350693E-6,-.494852E-6,
     &-.330680E-7,-.577241E-6,-.107591E-6,-.877215E-6,+.136268E-7/
      DATA (Y(I),I=169,186)/  -.837030E-7,+.156362E-5,+.450909E-7,
     &-.691157E-6,+.129643E-5,-.105347E-6,-.108080E-5,+.745489E-7,
     &-.152037E-5,-.885260E-6,+.381345E-7,-.163277E-5,+.334778E-6,
     &-.221182E-5,+.467095E-8,-.250010E-5,-.156800E-7, .000      /
      END
