      PROGRAM EX0604
      PARAMETER (IC=2)
      DIMENSION XX(IC),TTB(IC),TTC(IC),PPC(IC)
      DIMENSION PLK(IC,IC),Y1(IC),Y2(IC),Y3(IC),Y4(IC)
      DATA IP/1/
      DATA XX(1),TTB(1),TTC(1),PPC(1)/0.676,353.2,562.2,4.895E+6/
      DATA Y1(1),Y2(1),Y3(1),Y4(1)/-33.92,0.4739,-3.017E-4,7.13E-8/
      DATA XX(2),TTB(2),TTC(2),PPC(2)/0.324,398.8,568.95,2.49E+6/
      DATA Y1(2),Y2(2),Y3(2),Y4(2)/-6.096,0.7712,-4.195E-4,8.855E-8/
      DATA PLK(1,1),PLK(1,2),PLK(2,1),PLK(2,2)/1.,0.987,0.987,1./
      DO 300 I=1,2
      IF (I.EQ.1) T=590.
      IF (I.EQ.1) P=9.7E+6
      IF (I.EQ.2) T=470.
      IF (I.EQ.2) P=1.4E+6
      CALL PLOCKER
     &(IC,T,P,XX,TTB,TTC,PPC,PLK,Y1,Y2,Y3,Y4,A,TC,PC,W1,W2,W3,W4)
      CALL LEEKESL(T,P,TB,TC,PC,IC,IP,W1,W2,W3,W4,A,PS,Z,D,F,H,S,CV,CP)
      IF (I.EQ.1) H1=H
  300 IF (I.EQ.2) HT=(H1-H)/89.815
      WRITE (*,1040) HT
 1040 FORMAT(3X,'SPECIFIC ENTHALPY DIFFERENCE [KJ/KG] =',F5.1)
      END


      SUBROUTINE PLOCKER
     +(IC,T,P,XX,TTB,TTC,PPC,PLK,Y1,Y2,Y3,Y4,A,TC,PC,W1,W2,W3,W4)
C
C     The subroutine is to be used before subroutine LEEKESL, in
C     case of Multicomponent mixtures, to apply Mixing Rules to
C          1) the critical parameters
C          2) the acentric factor
C     based on the corresponding-states equations developed in
C     B.I.Lee & M.G.Kesler, AICHE J. 21:510 (1975)
C     - Since Acentric Factor is calculated here, the corresponding
C       part in the LEEKESL subroutine is not used.
C     - Perfect-gas Heat Capacity coefficients of mixture are
C       calculated as a mole fraction average of components coefficients
C       given as CP(I)=Y1(I)+Y2(I)*T+Y3(I)*(T**2)+Y4(I)*(T**3)
C     - The number of components is set at Switch IC.
C     - Data required for every component are: mole fraction XX(I),
C       normal boiling temperature, TTB(I) in [K], critical temperature,
C       TTC(I) in [K], critical pressure PPC(I) in [PA] and
C       binary interaction parameters, PLK(I,J).
C
      DIMENSION XX(IC),TTB(IC),TTC(IC),PPC(IC)
      DIMENSION PLK(IC,IC),Y1(IC),Y2(IC),Y3(IC),Y4(IC)
      DIMENSION VC(IC)
C
C     Acentric Factor & Critical Volume of component I
C
      A=0.0
      R=8.31451
      DO 200 I=1,IC
      TR=T/TTC(I)
      PR=P/PPC(I)
      TBR=TTB(I)/TTC(I)
      A1=LOG(0.10133E+6/PPC(I))-5.92714+6.09648/TBR+1.28862*LOG(TBR)
     +-0.169347*TBR**6
      A2=15.2518-15.6875/TBR-13.4721*LOG(TBR)+0.43577*TBR**6
      AM=A1/A2
      VC(I)=(0.2905-0.085*AM)*R*TTC(I)/PPC(I)
  200 A=A+XX(I)*AM
C
C     Mixture Critical Volume
C
      VVC=0.0
      DO 220 J=1,IC
      DO 220 K=1,IC
      VV=((VC(J)**(1./3.))+(VC(K)**(1./3.)))**3
  220 VVC=VVC+XX(J)*XX(K)*VV
      VVC=VVC/8.
C
C     Mixture Critical Temperature and Pressure
C
      TC=0.0
      DO 240 J=1,IC
      DO 240 K=1,IC
      VV=((VC(J)**(1./3.))+(VC(K)**(1./3.)))**(3.*0.25)
  240 TC=TC+XX(J)*XX(K)*VV*((TTC(J)*TTC(K))**0.5)*PLK(J,K)
      TC=TC/((8.*VVC)**0.25)
      PC=(0.2905-0.085*A)*R*TC/VVC
C
C     Mixture Heat Capacity Coefficients
C
      W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 260 I=1,IC
      W1=W1+XX(I)*Y1(I)
      W2=W2+XX(I)*Y2(I)
      W3=W3+XX(I)*Y3(I)
  260 W4=W4+XX(I)*Y4(I)
      RETURN
      END

      SUBROUTINE LEEKESL
     +(T,P,TB,TC,PC,IC,IP,W1,W2,W3,W4,A,PS,Z,D,F,H,S,CV,CP)
C
C     The subroutine calculates the following thermodynamic properties
C     of non-polar compounds, as a function of temperature, T in [K],
C     and pressure, P in [PA]:
C          1) Acentric Factor                   (A)  in [-]
C          2) Saturation Vapour Pressure       (PS)  in [MPA]
C          3) Compression Factor                (Z)  in [-]
C          4) Molar Density                     (D)  in [MOL/M3]
C          5) Fugacity                          (F)  in [PA]
C          6) Molar Entropy                     (H)  in [J/MOL]
C          7) Molar Enthalpy                    (S)  in [J/MOL/K]
C          8) Molar Isochoric Heat Capacity    (CV)  in [J/MOL/K]
C          9) Molar Isobaric Heat Capacity     (CP)  in [J/MOL/K]
C     based on the Corresponding-states equations developed in
C     B.I.Lee & M.G.Kesler, AICHE J. 21:510 (1975)
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IC = Number of components in mixture (Pure fluid=1)
C     - Other data required are: normal boiling temperature, TB in [K],
C       critical temperature, TC in [K], critical pressure, PC in [MPA].
C
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      DIMENSION ZF(2),FUG(2),ENTH(2),SS(2),CCV(2),CCP(2)
C
C     Perfect-gas Properties
C
      T0=273.15
      P0=0.10133E+6
      R=8.31451
      PGCP=W1+W2*T+W3*T*T+W4*(T**3)
      PGCV=PGCP-R
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     ++W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     ++W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Acentric Factor
C
      TR=T/TC
      PR=P/PC
      TBR=TB/TC
C
C     Check for Mixtures (IC>1), or Pure Fluid (IC=1)
C
      IF (IC.GT.1) GOTO 10
      A1=LOG(0.10133E+6/PC)-5.92714+6.09648/TBR+1.28862*LOG(TBR)
     +-0.169347*TBR**6
      A2=15.2518-15.6875/TBR-13.4721*LOG(TBR)+0.43577*TBR**6
      A=A1/A2
C
C     Saturation Vapour Pressure
C
   10 PS=PC*EXP(5.92714-6.09648/TR-1.28862*LOG(TR)+0.169347*TR**6
     ++A*(15.2518-15.6875/TR-13.4721*LOG(TR)+0.43577*TR**6))
C
C     Compression Factor
C
      DO 50 I=1,2
      IF (IP.EQ.1) THEN
      Z1=0.001
      Z2=0.010
   20 IF ((COMP(Z1,I,TR,PR)*COMP(Z2,I,TR,PR)).LT.0.0) GOTO 40
      Z1=Z2
      Z2=Z2+0.001
      GOTO 20
      ELSE
      Z1=1.1
      Z2=3.
   30 IF ((COMP(Z1,I,TR,PR)*COMP(Z2,I,TR,PR)).LT.0.0) GOTO 40
      Z2=Z1
      Z1=Z1-0.05
      GOTO 30
      ENDIF
   40 Z3=(Z1+Z2)/2.
      IF (ABS(Z1-Z2).LT.0.000001) GOTO 50
      IF ((COMP(Z3,I,TR,PR)*COMP(Z1,I,TR,PR)).LT.0.0) Z2=Z3
      IF ((COMP(Z3,I,TR,PR)*COMP(Z1,I,TR,PR)).GT.0.0) Z1=Z3
      GOTO 40
   50 ZF(I)=Z3
C
      DO 100 I=1,2
      B=B1(I)-B2(I)/TR-B3(I)/(TR**2)-B4(I)/(TR**3)
      C=C1(I)-C2(I)/TR+C3(I)/(TR**3)
      D=D1(I)+D2(I)/TR
      VR=ZF(I)*TR/PR
      EE=(BI(I)+1.+GA(I)/(VR**2))*EXP(-GA(I)/(VR**2))
      E=C4(I)*(BI(I)+1.-EE)/(2.*(TR**3)*GA(I))
C
C     Fugacity
C
      FU=EXP(ZF(I)-1.-LOG(ZF(I))+B/VR+C/(2.*(VR**2))+D/(5.*(VR**5))+E)
      FUG(I)=LOG10(FU)
C
C     Residual Entahlpy
C
      ENT1=(B2(I)+2.*B3(I)/TR+3.*B4(I)/(TR**2))/(TR*VR)
      ENT2=(C2(I)-3.*C3(I)/(TR**2))/(2.*TR*VR*VR)
      ENT3=D2(I)/(5.*TR*(VR**5))
      ENTH(I)=TR*(ZF(I)-1.-ENT1-ENT2+ENT3+3.*E)
C
C     Residual Entropy
C
      S1=(B1(I)+B3(I)/(TR**2)+2.*B4(I)/(TR**3))/VR
      S2=(C1(I)-2.*C3(I)/(TR**3))/(2.*VR*VR)
      S3=D1(I)/(5.*(VR**5))
      SS(I)=LOG(ZF(I))-S1-S2-S3+2.*E
C
C     Residual Isochoric Heat Capacity
C
      CV1=2.*(B3(I)+3.*B4(I)/TR)/(VR*TR*TR)
      CV2=3.*C3(I)/(VR*VR*(TR**3))
      CCV(I)=CV1-CV2-6.*E
C
C     Residual Isobaric Heat Capacity
C
      CP11=(B1(I)+B3(I)/(TR**2)+2.*B4(I)/(TR**3))/VR
      CP12=((C1(I)-2.*C3(I)/(TR**3))/(VR*VR))+(D1(I)/(VR**5))
      CP13=2.*C4(I)*(BI(I)+GA(I)/(VR*VR))*EXP(-GA(I)/(VR**2))
      CP13=CP13/(VR*VR*(TR**3))
      DPDT=(1.+CP11+CP12-CP13)/VR
      CP21=1.+2.*B/VR+3.*C/(VR*VR)+6.*D/(VR**5)
      CP22=3.*BI(I)+(GA(I)/(VR*VR))*(5.-2.*(BI(I)+GA(I)/(VR*VR)))
      CP22=(C4(I)/(VR*VR*(TR**3)))*CP22*EXP(-GA(I)/(VR*VR))
      DPDV=-(TR/(VR*VR))*(CP21+CP22)
  100 CCP(I)=CCV(I)-1.-TR*DPDT*DPDT/DPDV
C
C     Total Properties (& Compression Factor, Density, Fugacity)
C
      Z=ZF(1)+(A/0.3978)*(ZF(2)-ZF(1))
      D=P/(Z*R*T)
      F=P*10.**(FUG(1)+(A/0.3978)*(FUG(2)-FUG(1)))
      H=PGH+R*TC*(ENTH(1)+(A/0.3978)*(ENTH(2)-ENTH(1)))
      S=PGS+R*(SS(1)+(A/0.3978)*(SS(2)-SS(1)))
      CV=PGCV+R*(CCV(1)+(A/0.3978)*(CCV(2)-CCV(1)))
      CP=PGCP+R*(CCP(1)+(A/0.3978)*(CCP(2)-CCP(1)))
      IF (IC.GT.1) PS=99999999.9
      RETURN
      END

      FUNCTION COMP(Z,I,TR,PR)
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      B=B1(I)-B2(I)/TR-B3(I)/(TR**2)-B4(I)/(TR**3)
      C=C1(I)-C2(I)/TR+C3(I)/(TR**3)
      D=D1(I)+D2(I)/TR
      VR=Z*TR/PR
      ZZ=1.+B/VR+C/(VR**2)+D/(VR**5)
      COMP=Z-ZZ-((C4(I)/((TR**3)*(VR**2)))*(BI(I)+(GA(I)/(VR**2)))
     +*EXP(-GA(I)/(VR**2)))
      RETURN
      END

      BLOCK DATA
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      DATA B1,B2/0.1181193,0.2026579,0.2657280,0.3315110/
      DATA B3,B4/0.1547900,0.0276550,0.0303230,0.2034880/
      DATA C1,C2/0.0236744,0.0313385,0.0186984,0.0503618/
      DATA C3,C4/0.0000000,0.0169010,0.0427240,0.0415770/
      DATA D1,D2/0.155488E-4,0.48736E-4,0.623689E-4,0.0740336E-4/
      DATA BI,GA/0.65392,1.226,0.060167,0.03754/
      END
