      PROGRAM EX0605
      DATA T,P,TB,TC,PC,Y,IC/250.,2.E+6,247.,374.21,4.056E+6,0.195,1/
      DATA W1,W2,W3,W4/16.7803,0.28634,-2.2732E-4,1.13305E-7/
      DO 100 I=1,2
      IP=I
      T=T+FLOAT(I-1)*200.
      P=P+FLOAT(I-1)*8.E+6
      H1=H
      S1=S
      PS1=PS
      D1=D
  100 CALL LEEKESL
     &(T,P,TB,TC,PC,Y,IC,IP,W1,W2,W3,W4,A,PS,Z,D,F,H,S,CV,CP)
      WMOL=102.0314
      WRITE (*,1040) A,PS1*1.E-6,PS*1.E-6,(D1*WMOL/1000.),(D*WMOL/1000.)
     &,(H1/WMOL),(H/WMOL),(S1/WMOL),(S/WMOL)
 1040 FORMAT(5X,'> TEMPERATURE, PRESSURE                  ',
     &'      250K,2MPA    450K,10MPA',
     &/,5X,'ACENTRIC FACTOR                        [-] =',F17.3,
     &/,5X,'SATURATION VAPOUR PRESSURE           [MPA] =',2F11.1,
     &/,5X,'DENSITY                            [KG/M3] =',2F11.1,
     &/,5X,'ENTHALPY                           [KJ/KG] =',2F11.1,
     &/,5X,'ENTROPY                          [KJ/KG/K] =',2F12.2)
      WRITE (*,1060) (H-H1)/WMOL, (S-S1)/WMOL
 1060 FORMAT(1X,/,5X,'ENTHALPY DIFFERENCE                [KJ/KG] =',
     &F17.1,/5X,'ENTROPY DIFFERENCE               [KJ/KG/K] =',F18.2)
      END

      SUBROUTINE LEEKESL
     +(T,P,TB,TC,PC,Y,IC,IP,W1,W2,W3,W4,A,PS,Z,D,F,H,S,CV,CP)
C
C     The subroutine calculates the following thermodynamic properties
C     of non-polar compounds, as a function of temperature, T in [K],
C     and pressure, P in [PA]:
C          1) Acentric Factor                   (A)  in [-]
C          2) Saturation Vapour Pressure       (PS)  in [MPA]
C          3) Compression Factor                (Z)  in [-]
C          4) Molar Density                     (D)  in [MOL/M3]
C          5) Fugacity                          (F)  in [PA]
C          6) Molar Entropy                     (H)  in [J/MOL]
C          7) Molar Enthalpy                    (S)  in [J/MOL/K]
C          8) Molar Isochoric Heat Capacity    (CV)  in [J/MOL/K]
C          9) Molar Isobaric Heat Capacity     (CP)  in [J/MOL/K]
C     based on the Corresponding-states equations developed in
C     B.I.Lee & M.G.Kesler, AICHE J. 21:510 (1975)
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IC = Number of components in mixture (Pure fluid=1)
C     - Other data required are: normal boiling temperature, TB in [K],
C       critical temperature, TC in [K], critical pressure, PC in [MPA].
C
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      DIMENSION ZF(2),FUG(2),ENTH(2),SS(2),CCV(2),CCP(2)
C
C     Perfect-gas Properties
C
      T0=273.15
      P0=0.10133E+6
      R=8.31451
      PGCP=W1+W2*T+W3*T*T+W4*(T**3)
      PGCV=PGCP-R
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     ++W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     ++W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Acentric Factor
C
      TR=T/TC
      PR=P/PC
      TBR=TB/TC
C
C     Check for Mixtures (IC>1), or Pure Fluid (IC=1)
C
      IF (IC.GT.1) GOTO 10
      A1=LOG(0.10133E+6/PC)-5.92714+6.09648/TBR+1.28862*LOG(TBR)
     +-0.169347*TBR**6
      A2=15.2518-15.6875/TBR-13.4721*LOG(TBR)+0.43577*TBR**6
      A=A1/A2
C
C     Saturation Vapour Pressure
C
   10 PS=PC*EXP(5.92714-6.09648/TR-1.28862*LOG(TR)+0.169347*TR**6
     ++A*(15.2518-15.6875/TR-13.4721*LOG(TR)+0.43577*TR**6))
C
C     Compression Factor
C
      DO 50 I=1,2
      IF (IP.EQ.1) THEN
      Z1=0.001
      Z2=0.010
   20 IF ((COMP(Z1,I,TR,PR)*COMP(Z2,I,TR,PR)).LT.0.0) GOTO 40
      Z1=Z2
      Z2=Z2+0.01
      GOTO 20
      ELSE
      Z1=1.1
      Z2=1.2
   30 IF ((COMP(Z1,I,TR,PR)*COMP(Z2,I,TR,PR)).LT.0.0) GOTO 40
      Z2=Z1
      Z1=Z1-0.05
      GOTO 30
      ENDIF
   40 Z3=(Z1+Z2)/2.
      IF (ABS(Z1-Z2).LT.0.000001) GOTO 50
      IF ((COMP(Z3,I,TR,PR)*COMP(Z1,I,TR,PR)).LT.0.0) Z2=Z3
      IF ((COMP(Z3,I,TR,PR)*COMP(Z1,I,TR,PR)).GT.0.0) Z1=Z3
      GOTO 40
   50 ZF(I)=Z3
C
      DO 100 I=1,2
      B=B1(I)-B2(I)/TR-B3(I)/(TR**2)-B4(I)/(TR**3)
      C=C1(I)-C2(I)/TR+C3(I)/(TR**3)
      D=D1(I)+D2(I)/TR
      VR=ZF(I)*TR/PR
      EE=(BI(I)+1.+GA(I)/(VR**2))*EXP(-GA(I)/(VR**2))
      E=C4(I)*(BI(I)+1.-EE)/(2.*(TR**3)*GA(I))
C
C     Fugacity
C
      FU=EXP(ZF(I)-1.-LOG(ZF(I))+B/VR+C/(2.*(VR**2))+D/(5.*(VR**5))+E)
      FUG(I)=LOG10(FU)
C
C     Residual Entahlpy
C
      ENT1=(B2(I)+2.*B3(I)/TR+3.*B4(I)/(TR**2))/(TR*VR)
      ENT2=(C2(I)-3.*C3(I)/(TR**2))/(2.*TR*VR*VR)
      ENT3=D2(I)/(5.*TR*(VR**5))
      ENTH(I)=TR*(ZF(I)-1.-ENT1-ENT2+ENT3+3.*E)
C
C     Residual Entropy
C
      S1=(B1(I)+B3(I)/(TR**2)+2.*B4(I)/(TR**3))/VR
      S2=(C1(I)-2.*C3(I)/(TR**3))/(2.*VR*VR)
      S3=D1(I)/(5.*(VR**5))
      SS(I)=LOG(ZF(I))-S1-S2-S3+2.*E
C
C     Residual Isochoric Heat Capacity
C
      CV1=2.*(B3(I)+3.*B4(I)/TR)/(VR*TR*TR)
      CV2=3.*C3(I)/(VR*VR*(TR**3))
      CCV(I)=CV1-CV2-6.*E
C
C     Residual Isobaric Heat Capacity
C
      CP11=(B1(I)+B3(I)/(TR**2)+2.*B4(I)/(TR**3))/VR
      CP12=((C1(I)-2.*C3(I)/(TR**3))/(VR*VR))+(D1(I)/(VR**5))
      CP13=2.*C4(I)*(BI(I)+GA(I)/(VR*VR))*EXP(-GA(I)/(VR**2))
      CP13=CP13/(VR*VR*(TR**3))
      DPDT=(1.+CP11+CP12-CP13)/VR
      CP21=1.+2.*B/VR+3.*C/(VR*VR)+6.*D/(VR**5)
      CP22=3.*BI(I)+(GA(I)/(VR*VR))*(5.-2.*(BI(I)+GA(I)/(VR*VR)))
      CP22=(C4(I)/(VR*VR*(TR**3)))*CP22*EXP(-GA(I)/(VR*VR))
      DPDV=-(TR/(VR*VR))*(CP21+CP22)
  100 CCP(I)=CCV(I)-1.-TR*DPDT*DPDT/DPDV
C
C     Total Properties (& Compression Factor, Density, Fugacity)
C
      Z=ZF(1)+(A/0.3978)*(ZF(2)-ZF(1))
      D=P/(Z*R*T)
      F=P*10.**(FUG(1)+(A/0.3978)*(FUG(2)-FUG(1)))
      H=PGH+R*TC*(ENTH(1)+(A/0.3978)*(ENTH(2)-ENTH(1)))
      S=PGS+R*(SS(1)+(A/0.3978)*(SS(2)-SS(1)))
      CV=PGCV+R*(CCV(1)+(A/0.3978)*(CCV(2)-CCV(1)))
      CP=PGCP+R*(CCP(1)+(A/0.3978)*(CCP(2)-CCP(1)))
      IF (IC.GT.1) PS=99999999.9
C
C     Correction for Polar compounds
C
      CALL WUSTIEL(IP,TR,PR,ZW,HW,SW)
      Z2=ZW-(ZF(1)+0.344*(ZF(2)-ZF(1))/0.3978)
      Z=Z+Y*Z2
      D=P/(Z*R*T)
      ENTH2=HW*TR-(ENTH(1)+0.344*(ENTH(2)-ENTH(1))/0.3978)
      H=H+(Y*ENTH2*R*TC)
      S2=SW-(SS(1)+0.344*(SS(2)-SS(1))/0.3978)
      S=S+(Y*S2*R)
      IF (ABS(S2).GT.2.0.OR.ABS(ENTH2).GT.1.) WRITE (*,1050)
 1050 FORMAT(12X,'>>> TR OR PR OUT OF RANGE, NO CONVERGENCE <<<')
      RETURN
      END


      FUNCTION COMP(Z,I,TR,PR)
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      B=B1(I)-B2(I)/TR-B3(I)/(TR**2)-B4(I)/(TR**3)
      C=C1(I)-C2(I)/TR+C3(I)/(TR**3)
      D=D1(I)+D2(I)/TR
      VR=Z*TR/PR
      ZZ=1.+B/VR+C/(VR**2)+D/(VR**5)
      COMP=Z-ZZ-((C4(I)/((TR**3)*(VR**2)))*(BI(I)+(GA(I)/(VR**2)))
     +*EXP(-GA(I)/(VR**2)))
      RETURN
      END


c      BLOCK DATA
c      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
c     +D1(2),D2(2),BI(2),GA(2)
c      DATA B1,B2/0.1181193,0.2026579,0.2657280,0.3315110/
c      DATA B3,B4/0.1547900,0.0276550,0.0303230,0.2034880/
c      DATA C1,C2/0.0236744,0.0313385,0.0186984,0.0503618/
c      DATA C3,C4/0.0000000,0.0169010,0.0427240,0.0415770/
c      DATA D1,D2/0.155488E-4,0.48736E-4,0.623689E-4,0.0740336E-4/
c      DATA BI,GA/0.65392,1.226,0.060167,0.03754/
c      END


      SUBROUTINE WUSTIEL(IP,TR,PR,ZW,HW,SW)
C
C     The subroutine calculates the following corrections to the
C     thermodynamic properties of polar compounds, as a function of
C     Reduced Temperature TR, and Reduced Pressure PR:
C           1) Compression Factor Correction    (ZW)  in       [-]
C           2) Molar Enthalpy Correction        (HW)  in   [J/MOL]
C           3) Molar Entropy Correction         (SW)  in [J/MOL/K]
C     based on the Corresponding-states equations developed in
C     G.Z.A. Wu and L.I. Stiel, AIChE J 31:1632 (1985)
C
C     - The subroutine must be called from LEEKESL as described in
C       Section 5.5.3.
C     - Phase of fluid (IP) is specified in LEEKESL routine
C     - Parameter Y (polarity factor) must be entered from the
C       calling program
C
      COMMON /EQCONZW/ A(10,7),TAJ(7),RAJ(7)
      T=647.29*TR
      P=22.088*PR
C
C     Compression Factor Correction
C
      IF (IP.EQ.1) THEN
      Z1=0.001
      Z2=0.010
   21 IF ((COMPZW(Z1,T,P)*COMPZW(Z2,T,P)).LT.0.0) GOTO 41
      Z1=Z2
      Z2=Z2+0.001
      GOTO 21
      ELSE
      Z1=1.1
      Z2=3.
   31 IF ((COMPZW(Z1,T,P)*COMPZW(Z2,T,P)).LT.0.0) GOTO 41
      Z2=Z1
      Z1=Z1-0.05
      GOTO 31
      ENDIF
   41 Z3=(Z1+Z2)/2.
      IF (ABS(Z1-Z2).LT.0.000001) GOTO 51
      IF ((COMPZW(Z3,T,P)*COMPZW(Z1,T,P)).LT.0.0) Z2=Z3
      IF ((COMPZW(Z3,T,P)*COMPZW(Z1,T,P)).GT.0.0) Z1=Z3
      GOTO 41
   51 ZW=Z3
C
C     Molar Enthalpy and Entropy Correction
C
      TA=1000./T
      RW=P/(0.46151*ZW*T)
      Q=0.
      DQT=0.
      DO 71 J=1,7
      EX=(EXP(-4.8*RW))*(A(9,J)+A(10,J)*RW)
      QS=0.
      DO 61 I=1,8
   61 QS=QS+A(I,J)*((RW-RAJ(J))**(I-1))
      Q=Q+((TA-TAJ(J))**(J-2))*(QS+EX)*(TA-TAJ(1))
   71 DQT=DQT+((TA-TAJ(J))**(J-2))*(QS+EX)+
     &(FLOAT(J-2)*(TA-TAJ(J))**(J-3))*(QS+EX)*(TA-TAJ(1))
      HW=(ZW-1.)+(RW*TA*DQT)
      SW=LOG(ZW)-(RW*Q)+(RW*TA*DQT)
      RETURN
      END


      FUNCTION COMPZW(Z,T,P)
      COMMON /EQCONZW/ A(10,7),TAJ(7),RAJ(7)
      TA=1000./T
      RW=P/(0.46151*Z*T)
      Q=0.
      DQ=0.
      DO 111 J=1,7
      EX=(EXP(-4.8*RW))*(A(9,J)+A(10,J)*RW)
      DEX=(EXP(-4.8*RW))*(-4.8*A(9,J)-4.8*A(10,J)*RW+A(10,J))
      QS=0.
      DQS=0.
      DO 101 I=1,8
      QS=QS+A(I,J)*((RW-RAJ(J))**(I-1))
  101 DQS=DQS+(FLOAT(I-1)*A(I,J)*((RW-RAJ(J))**(I-2)))
      Q=Q+((TA-TAJ(J))**(J-2))*(QS+EX)*(TA-TAJ(1))
  111 DQ=DQ+((TA-TAJ(J))**(J-2))*(DQS+DEX)*(TA-TAJ(1))
      COMPZW=Z-1.-(RW*Q)-(RW*RW*DQ)
      RETURN
      END


      BLOCK DATA
      COMMON /EQCONZW/ A(10,7),TAJ(7),RAJ(7)
      COMMON /EQCON/ B1(2),B2(2),B3(2),B4(2),C1(2),C2(2),C3(2),C4(2),
     +D1(2),D2(2),BI(2),GA(2)
      DATA B1,B2/0.1181193,0.2026579,0.2657280,0.3315110/
      DATA B3,B4/0.1547900,0.0276550,0.0303230,0.2034880/
      DATA C1,C2/0.0236744,0.0313385,0.0186984,0.0503618/
      DATA C3,C4/0.0000000,0.0169010,0.0427240,0.0415770/
      DATA D1,D2/0.155488E-4,0.48736E-4,0.623689E-4,0.0740336E-4/
      DATA BI,GA/0.65392,1.226,0.060167,0.03754/
      DATA TAJ/1.544912,2.5,2.5,2.5,2.5,2.5,2.5/
      DATA RAJ/0.634,1.,1.,1.,1.,1.,1./
      DATA (A(1,K),K=1,7)/29.492937,-5.198586,6.8335354,-0.1564104,
     &-6.3972405,-3.9661401,-0.69048554/
      DATA (A(2,K),K=1,7)/-132.13917,7.7779182,-26.149751,-0.72546108,
     &26.409282,15.453061,2.7407416/
      DATA (A(3,K),K=1,7)/274.64632,-33.301902,65.326396,-9.2734289,
     &-47.740374,-29.142470,-5.1028070/
      DATA (A(4,K),K=1,7)/-360.93828,-16.254622,-26.181978,4.3125840,
     &56.323130,29.568796,3.9636085/
      DATA (A(5,K),K=1,7)/342.18431,-177.31074,0.,0.,0.,0.,0./
      DATA (A(6,K),K=1,7)/-244.50042,127.48742,0.,0.,0.,0.,0./
      DATA (A(7,K),K=1,7)/155.18535,137.46153,0.,0.,0.,0.,0./
      DATA (A(8,K),K=1,7)/5.9728487,155.97836,0.,0.,0.,0.,0./
      DATA (A(9,K),K=1,7)/-410.30848,337.31180,-137.46618,6.7874983,
     &136.87317,79.847970,13.041253/
      DATA (A(10,K),K=1,7)/-416.05860,-209.88866,-733.96848,10.401717,
     &645.81880,399.17570,71.531353/
      END
