      PROGRAM EX0706
      CHARACTER*10 FNAM(3)
      DATA X1,T,GAM1,GAM2,V12,A12/0.736,400.,1.0224,1.3983,0.494,0.2/
      DATA GUES12,GUES21/0.5,0.5/
      DATA FNAM/'WILSON    ','T-K-WILSON','NRTL      '/
      WRITE(*,1000) X1,V12,T,GAM1,GAM2
 1000 FORMAT(1X,'@ X1       =',F7.4,3X,'V12=',F7.4,3X,'T [K]=',
     &F7.2,/3X,'G1(EXPT)=',F7.4,/3X,'G2(EXPT)=',F7.4,/)
C
C     WILSON (IOPT=1)
C
      CALL WILSPAR(T,X1,GAM1,GAM2,V12,GUES12,GUES21,SL12R,SL21R,1)
      WRITE(*,1100) FNAM(1),SL12R,SL21R
 1100 FORMAT(3X,A10,' PARAMETERS:',3X,'L12/R=',F8.2,4X,
     &'L21/R =',2X,F8.2)
C
C     T-K-WILSON (IOPT=2)
C
      CALL WILSPAR(T,X1,GAM1,GAM2,V12,GUES12,GUES21,SL12R,SL21R,2)
      WRITE(*,1100) FNAM(2),SL12R,SL21R
C
C     NRTL
C
      CALL NRTLPAR(T,X1,GAM1,GAM2,A12,GUES12,GUES21,G12R,G21R)
      WRITE(*,1200) FNAM(3),G12R,G21R,A12
 1200 FORMAT(3X,A10,' PARAMETERS:',1X,'T12/R=',F8.2,2X,
     &'T21/R=',2X,F8.2,1X,'(A12 =',F7.4,')')
      END



      SUBROUTINE WILSPAR
     &(T,X1,GAM1,GAM2,V12,SL12,SL21,SL12R,SL21R,IOPT)
C
C     The subroutine calculates Wilson (IOPT=1) and T-K-Wilson 
C     parameters  SLij=Vij*exp[-SLIJR/T]
C     for given activity coefficients GAM1, GAM2 at molefraction X1,
C     and temperature T [K].
C     - Guess starting values of parameters SL12, SL21 are required
C       (usually both at 0.5)
C
      X2=1.-X1
      IF (IOPT.EQ.1) V12E=1.
      IF (IOPT.EQ.2) V12E=V12
      V21E=1./V12E
C
C     WILSON & T-K-WILSON EQUATIONS
C
   70 F1=LOG(GAM1)-LOG((X1+X2*V12E)/(X1+X2*SL12))
      F1=F1-X1*((1./(X1+X2*V12E))-(1./(X1+X2*SL12)))
      F1=F1-X2*((V21E/(X1*V21E+X2))-(SL21/(X1*SL21+X2)))
      F2=LOG(GAM2)-LOG((X1*V21E+X2)/(X1*SL21+X2))
      F2=F2-X1*((V12E/(X1+X2*V12E))-(SL12/(X1+X2*SL12)))
      F2=F2-X2*((1./(X1*V21E+X2))-(1./(X1*SL21+X2)))
C
C     DERIVATIVES
C
      F1L12=SL12*(X2/(X1+X2*SL12))**2
      F1L21=(X2/(X2+X1*SL21))**2
      F2L12=(X1/(X1+SL12*X2))**2
      F2L21=SL21*(X1/(X2+SL21*X1))**2
C
C     SOLUTION
C
      D=F2L12*F1L21-F1L12*F2L21
      H=(F1*F2L21-F2*F1L21)/D
      G=(F1L12*F2-F2L12*F1)/D
      SL12=SL12+H
      SL21=SL21+G
      CHK=ABS(H/SL12)+ABS(G/SL21)
      IF (CHK.GE.0.0001) GOTO 70
C
C     SMALL GREEK LAMDA(I,J)/R
C
      SL12R=-T*LOG(SL12/V12)
      SL21R=-T*LOG(SL21*V12)
      RETURN
      END



      SUBROUTINE NRTLPAR
     &(T,X1,GAM1,GAM2,A12,T12,T21,G12R,G21R)
C
C     The subroutine calculates NRTL parametes G12R=g12/R and 
C     G21R=g21/R
C           Gij=exp(-aij*Tij)  where  Tij=(gij/R)/T
C     for given activity coefficients GAM1, GAM2 at molefraction X1,
C     temperature T [K] and NRTL parameter A12.
C     - Guess starting values of parameters T12, T21 are required
C       (usually both at 0.5 for A12=0.2)
C
      X2=1.-X1
C
C     NRTL EQUATIONS
C
   60 G12=EXP(-A12*T12)
      G21=EXP(-A12*T21)
      GX1=X1+X2*G21
      GX2=X2+X1*G12
      F1=LOG(GAM1)-X2*X2*T21*(G21/GX1)**2-X2*X2*T12*G12/(GX2**2)
      F2=LOG(GAM2)-X1*X1*T12*(G12/GX2)**2-X1*X1*T21*G21/(GX1**2)
C
C     DERIVATIVES
C
      F1L12=(1.-A12*T12)+2.*G12*A12*(X1*X2+X1*X1)/(GX2**2)
      F1L12=-X2*X2*G12*F1L12/(GX2**2)
      F1L21=(G21-2.*A12*T12)+2.*G21*G21*A12*(X1*X2+X2*X2)/(GX1**2)
      F1L21=-X2*X2*G21*F1L21/(GX1**2)
      F2L12=(1.-A12*T21)+2.*G21*A12*(X2*X1+X2*X2)/(GX1**2)
      F2L12=-X1*X1*G21*F2L12/(GX1**2)
      F2L21=(G12-2.*A12*T21)+2.*G12*G12*A12*(X2*X1+X1*X1)/(GX2**2)
      F2L21=-X1*X1*G12*F2L21/(GX2**2)
C
C     SOLUTION
C
      D=F2L12*F1L21-F1L12*F2L21
      H=(F1*F2L21-F2*F1L21)/D
      G=(F1L12*F2-F2L12*F1)/D
      T12=T12+H
      T21=T21+G
      CHK=ABS(H/T12)+ABS(G/T21)
      IF (CHK.GE.0.0001) GOTO 60
      G12R=T12*T
      G21R=T21*T
      RETURN
      END
