      PROGRAM EX0807
      PARAMETER (IS=3)
      DIMENSION TC(IS),PC(IS),OMEG(IS),Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION X(IS),Y(IS),Z(IS),FUGV(IS),FUGL(IS),BK(IS,IS)
      DATA TC(1),PC(1),OMEG(1)/305.33,4.871E+6,0.099/
      DATA TC(2),PC(2),OMEG(2)/425.25,3.792E+6,0.199/
      DATA TC(3),PC(3),OMEG(3)/469.80,3.375E+6,0.251/
      DATA Y1(1),Y2(1),Y3(1),Y4(1)/5.409,0.1781,-6.9385E-5,8.713E-9/
      DATA Y1(2),Y2(2),Y3(2),Y4(2)/9.487,0.3313,-1.108E-4,-2.822E-9/
      DATA Y1(3),Y2(3),Y3(3),Y4(3)/-3.626,0.4873,-2.580E-4,5.305E-8/
      DATA BK(1,1),BK(1,2),BK(1,3)/0.,0.01,0.01/
      DATA BK(2,1),BK(2,2),BK(2,3)/0.01,0.,0.01/
      DATA BK(3,1),BK(3,2),BK(3,3)/0.01,0.01,0./
      DATA TF,IF,Z/310.,1,0.30,0.30,0.40/
      DATA P,WLO,WHI/0.7E+6,285.,305./

      CALL FLASHHP
     &(IS,P,WLO,WHI,TF,IF,Z,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,X,Y,T,FUGL,FUGV,
     &BETA)

      WRITE (*,1100) T,P*1.E-6
 1100 FORMAT(3X,/,4X,'>>>  T=',F7.2,'K  &  P=',F6.3,'MPA',/,4X,70('-'),
     &/,12X,'Z(I)',5X,'X(I)',5X,'Y(I)',8X,'FUG.LIQ.',4X,'FUG.VAP.',
     &8X,'K')
      DO 20 J=1,IS
   20 WRITE (*,1200) J,Z(J),X(J),Y(J),FUGL(J),FUGV(J),FUGL(J)/FUGV(J)
 1200 FORMAT(4X,'(',I1,') ',3F9.4,2X,2(2X,F10.5),3X,F10.5)
      WRITE (*,1300) (1-BETA),BETA
 1300 FORMAT(3X,'PHASE',/,3X,'KMOLES  1.000  ',2(F7.3,2X))
      END


      SUBROUTINE FLASHHP
     &(IS,P,WLO,WHI,TF,IF,Z,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,X,Y,T,FUGL,FUGV,
     &BETA)
C
C     The subroutine performs an Adiabatic Flash calculation at given P
C     in [Pa] for a mixture feed Z(I) of IS components at temperature
C     TF [K] at specified phase (IF=1 liquid,IF=2 vapour).
C        Method adopted is based on the calculation of the liquid and
C     vapour partial fugacity coefficients from the P&R EoS.
C     -  Addiotional input to the subroutine are critical constants
C     TC(I) in [K], and PC(I) in [Pa], acentric factor OMEG(I),
C     and ideal heat capacity coefficients Y1(I),Y2(I),Y3(I),Y4(I).
C        Also first and last value, WLO and WHI, for the temperature
C     must be provided, as well as the PR binary interaction parameters
C     BK(IS,IS).
C     -  Output includes the liquid and vapour molefractions, X(I), Y(I)
C     the flash temperature T [K], the vapourised fraction BETA and the
C     liquid and vapour partial fugacity coefficients FUGL(I), FUGV(I).
C
      DIMENSION TC(IS),PC(IS),OMEG(IS),Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION X(IS),Y(IS),Z(IS),FUGL(IS),FUGV(IS),BK(IS,IS),EK(10)
      R=8.3145
C
C     INITIAL FLASH TEMPERATURE
C
      CALL PR(TF,P,IF,IS,Z,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,HF,SF,FUGL)
   10 T=(WLO+WHI)/2.
C
C     SET X(I), Y(I) EQUAL TO Z(I) TO START
C
      DO 100 I=1,IS
      X(I)=Z(I)
  100 Y(I)=Z(I)
C
C     CALCULATE FIRST EK, AND ENTHALPIES OF 2 PHASES
C
   20 CALL PR(T,P,2,IS,Y,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,HV,SV,FUGV)
      CALL PR(T,P,1,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,HL,SL,FUGL)
      CHKT=ABS(FUGV(1)-FUGL(1))*100./FUGV(1)
      IF (CHKT.GT.0.001) GOTO 30
      WRITE (*,*) '  NO CONVERGENCE - TRY CLOSER TEMPERATURE LIMITS'
      STOP
   30 SY=0.
      DO 120 I=1,IS
  120 EK(I)=FUGL(I)/FUGV(I)
C
C     CALCULATE BETA & FIRST X(I), Y(I)
C
   40 BETA=1.
   50 ST=0.
      SB=0.
      DO 140 I=1,IS
      ST=ST+Z(I)/(1.+BETA*(EK(I)-1.))
  140 SB=SB+Z(I)*(EK(I)-1.)/((1.+BETA*(EK(I)-1.))**2)
      BETAN=BETA+(ST-1.)/SB
      IF (ABS(BETAN-BETA).LE.0.000001) GOTO 60
      BETA=BETAN
      GOTO 50
   60 SYDIF=0.
      DO 160 I=1,IS
  160 SYDIF=SYDIF+ABS(X(I)*EK(I)-Y(I))
      SX=0.
      SY=0.
      DO 180 I=1,IS
      X(I)=Z(I)/(1.+BETA*(EK(I)-1.))
      Y(I)=EK(I)*X(I)
      SX=SX+X(I)
  180 SY=SY+Y(I)
      DO 200 I=1,IS
      X(I)=X(I)/SX
  200 Y(I)=Y(I)/SY
      IF (SYDIF.GT.0.00001) GO TO 20
C
C     TEMPERATURE ITERATION
C
      HVL=BETA*HV+(1.-BETA)*HL
      HDIF=HF-HVL
      WRITE (*,*) T
      IF (ABS(HDIF).LE.0.1) GOTO 70
      IF (ABS(HF).GT.ABS(HVL)) WHI=T
      IF (ABS(HF).LT.ABS(HVL)) WLO=T
      GOTO 10
   70 RETURN
      END


      SUBROUTINE PR
     &(T,P,IP,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Density                           (D)  in [MOL/M3]
C          2) Enthalpy                          (H)  in [J/MOL]
C          3) Entropy                           (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the Peng & Robinson EoS, shown in Table 6.6
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IS = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       MoleFraction X (-), Critical Temperature, TC in [K], Critical
C       Pressure in [PA], and Acentric factor, OMEG.
C     - Binary interaction parameters BK(I,J) must also be specified.
C
      DIMENSION X(IS),TC(IS),PC(IS),OMEG(IS),Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION BK(IS,IS),FUG(IS)
      DIMENSION A(10),B(10),FM(10),ALP(10),AS(10)
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters calculation AT,BT,CT,DT &  AST,ALP
C
      BT=0.
      DO 100 I=1,IS
      AS(I)=0.45724*R*R*TC(I)*TC(I)/PC(I)
      FM(I)=0.37464+1.54226*OMEG(I)-0.26992*(OMEG(I)**2)
      ALP(I)=(1.+FM(I)*(1.-(T/TC(I))**.5))**2
      A(I)=0.45724*ALP(I)*(P/PC(I))/((T/TC(I))**2)
      B(I)=0.0778*(P/PC(I))/(T/TC(I))
  100 BT=BT+X(I)*B(I)
      AAT=0.
      AT=0.
      DT=0.
      DO 200 I=1,IS
      DO 200 J=1,IS
      BIP=(1.-BK(I,J))
      AAT=AAT+X(I)*X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      AT=AT+X(I)*X(J)*BIP*((A(I)*A(J))**.5)
  200 DT=DT+X(I)*X(J)*FM(J)*BIP*((AS(I)*ALP(I)*AS(J)*(T/TC(J)))**.5)
C
C     Density Calculation
C
      IF (IP.EQ.1) Z=0.01
      IF (IP.EQ.2) Z=0.8
  300 F=(Z**3)-(1.-BT)*Z*Z+(AT-3.*(BT**2)-2.*BT)*Z-(AT*BT-BT*BT-BT**3)
      F1=(3.*Z*Z)-(2.*Z*(1.-BT))+(AT-3.*(BT**2)-2.*BT)
      H=F/F1
      Z=Z-H
      IF (ABS(H/Z).LE.0.0001) GOTO 400
      GOTO 300
  400 D=P/(Z*R*T)
C
C     Perfect-gas Enthalpy & Entropy
C
      W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IS
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Enthalpy & Entropy
C
      DH=1.-Z+(AT/(2.828*BT))*(1.+DT/AAT)*LOG((Z+2.414*BT)/(Z-0.414*BT))
      H=(PGH-R*T*DH)
      DS=LOG(Z-BT)-(BT*DT/(2.828*AT*AAT))*LOG((Z+2.414*BT)/
     &(Z-0.414*BT))
      S=PGS+(R*DS)
C
C     Partial Fugacities
C
      DO 700 I=1,IS
      XAAT=0.
      DO 800 J=1,IS
      BIP=(1.-BK(I,J))
  800 XAAT=XAAT+X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      FILN=(B(I)*(Z-1.)/BT)-LOG(Z-BT)+ (AT/(2.828*BT))*((B(I)/BT)-
     &(2.*XAAT/AAT))*LOG((Z+2.414*BT)/(Z-0.414*BT))
  700 FUG(I)=EXP(FILN)
      RETURN
      END
