      PROGRAM EX0808
      PARAMETER (IS=3)
      DIMENSION X1(IS),X2(IS),Z(IS),XST(IS)
      DIMENSION U(IS,IS),R(IS),Q(IS),QPR(IS),G1(IS),G2(IS)
      DATA (U(1,I),I=1,IS)/  0.   ,115.14,814.64/
      DATA (U(2,I),I=1,IS)/ 29.585,  0.  ,317.21/
      DATA (U(3,I),I=1,IS)/334.88 ,-22.287, 0.  /
      DATA R/3.9228,2.5735,0.9200/
      DATA Q/2.968,2.336,1.400/
      DATA QPR/2.968,2.336,1.400/
      DATA T,P/283.15,0.101E+6/
      WRITE (*,1000)
 1000 FORMAT(5X,'X1(1)',5X,'X1(2)',5X,'X1(3)',8X,'X2(1)',5X,'X2(2)',
     &5X,'X2(3)',7X,'BETA')
      DO 20 I=1,7
      Z(1)=0.1*FLOAT(I)
      Z(2)=(1.-Z(1))/2.
      Z(3)=Z(2)
C
C     Set initial values for X1 phase, XST, and BETA, BST
C
      XST(1)=0.1*FLOAT(I)
      XST(2)=0.9-XST(1)
      XST(3)=0.1
      BST=0.6
      CALL FLASH2L(IS,P,T,X1,X2,XST,BST,Z,U,R,Q,QPR,BETA)
   20 WRITE (*,1100) X1(1),X1(2),X1(3),X2(1),X2(2),X2(3),BETA
 1100 FORMAT(2X,3(F8.4,2X),3X,3(F8.4,2X),F10.4)
      END


      SUBROUTINE FLASH2L(IS,P,T,X1,X2,XST,BST,Z,U,R,Q,QPR,BETA)
C
C     The subroutine calculates equilibrium composition of the two
C     liquid phases, X1(I) and X2(I), of a ternary mixture (IS=3) of
C     composition Z(I). The amount in phase 1, BETA, is also calculated.
C        Method adopted is based on the calculation of the distribution
C     constant via the activity coefficients of the two phases. The
C     activity coefficients are calculated by UNIQUAC.
C     -  Input to the subroutine is the pressure P in [Pa], the tempera-
C     ture T in [K], initial values of molefractions of phase 1, XST(I),
C     and the fraction of ammount in phase 1, BST, molefractions of the
C     total mixture Z(I), and UINQUAC parameters, U, R, Q and QPR.
C     -  Output of the subroutine are the molefractions of the two
C     liquid phases, X1(I) and X2(I) and the fraction of ammoun in phase
C     1, BETA.
C
      DIMENSION X1(IS),X2(IS),Z(IS),XST(IS)
      DIMENSION U(IS,IS),R(IS),Q(IS),QPR(IS),G1(3),G2(3)
C
C     Set initial X1(I) (=XST), BETA (=BST)
C
      DO 100 I=1,IS
  100 X1(I)=XST(I)
      BETA=BST
C
C     Calculate X2(I), & activity coefficients
C
      DO 120 I=1,IS
  120 X2(I)=(Z(I)-BETA*X1(I))/(1.-BETA)
   20 BETAPR=BETA
      CALL UNIQUAC(IS,T,X1,R,Q,QPR,U,G1)
      CALL UNIQUAC(IS,T,X2,R,Q,QPR,U,G2)
C
C     Calculate BETA and X1(I), X2(I)
C
   40 ST=0.
      SB=0.
      DO 140 I=1,IS
      DK=G1(I)/G2(I)
      ST=ST+Z(I)/(BETA+DK*(1.-BETA))
  140 SB=SB+Z(I)*(1.-DK)/((BETA+DK*(1.-BETA))**2)
      BETAN=BETA+(ST-1.)/SB
      IF (ABS(BETAN-BETA).LE.0.000001) GOTO 60
      BETA=BETAN
      GOTO 40
   60 SX=0.
      SY=0.
      DO 160 I=1,IS
      DK=G1(I)/G2(I)
      X1(I)=Z(I)/(BETA+DK*(1.-BETA))
  160 X2(I)=DK*X1(I)
C
C     Iteration for BETA
C
      IF (ABS(BETA-BETAPR).GE.0.00001) GOTO 20
      RETURN
      END


      SUBROUTINE UNIQUAC(IS,T,X,R,Q,QPR,U,G)
C
C     The subroutine calculates activity coefficients G(I) from the
C     UNIQUAC equations, for a mixture at temperature T [K], of IS
C     components each of molefraction X(I).
C     Other input required is
C     - R(I)   : r(i), volume parameter of component i
C     - Q(I)   : q(i), area parameter of component i
C     - QPR(I) : q'(i), area parameter for water and alcohols
C                       for all other compounds equal to q(i), and
C     - U(I,J) : u(I,J)/R
C
      DIMENSION U(IS,IS),X(IS),R(IS),Q(IS),QPR(IS),G(IS)
      DIMENSION PHI(5),THI(5),THIPR(5),WL(5),TA(5,5)
      SPHI=0.
      STHI=0.
      STHIPR=0.
      DO 100 J=1,IS
      SPHI=SPHI+X(J)*R(J)
      STHI=STHI+X(J)*Q(J)
  100 STHIPR=STHIPR+X(J)*QPR(J)
      DO 120 I=1,IS
      PHI(I)=X(I)*R(I)/SPHI
      THI(I)=X(I)*Q(I)/STHI
      THIPR(I)=X(I)*QPR(I)/STHIPR
  120 WL(I)=5.*(R(I)-Q(I))-(R(I)-1.)
      DO 140 I=1,IS
      DO 140 J=1,IS
  140 TA(I,J)=EXP(-U(I,J)/T)
C
C     Configurational part
C
      DO 220 I=1,IS
      SXL=0.
      STHTJI=0.
      DO 160 J=1,IS
      SXL=SXL+X(J)*WL(J)
  160 STHTJI=STHTJI+TA(J,I)*THIPR(J)
      GC=LOG(PHI(I)/X(I))+5.*Q(I)*LOG(THI(I)/PHI(I))
      GC=GC+WL(I)-PHI(I)*SXL/X(I)
C
C     Residual part
C
      FS=0.
      DO 200 J=1,IS
      STHT=0.
      DO 180 K=1,IS
  180 STHT=STHT+THIPR(K)*TA(K,J)
  200 FS=FS+THIPR(J)*TA(I,J)/STHT
      GR=QPR(I)*(1.-LOG(STHTJI)-FS)
C
C     Activity coefficient
C
  220 G(I)=EXP(GC+GR)
      RETURN
      END
